import { SourceMap } from './ValidationService';
import { StringMap } from '@ethereum-sourcify/core';
/**
 * Abstraction of a checked solidity contract. With metadata and source (solidity) files.
 * The info property contains the information about compilation or errors encountered while validating the metadata.
 */
export default class CheckedContract {
    /** Object containing contract metadata keys and values. */
    metadata: any;
    /** SourceMap mapping the original compilation path to PathContent. */
    solidity: StringMap;
    /** Object containing the information about missing source files. */
    missing: any;
    /** Contains the invalid source files. */
    invalid: StringMap;
    /**
     * Contains the message about compilation or errors encountered while validating the metadata.
     * Should be without a trailing newline.
     */
    info: string;
    /** Object containing input for solc when used with the --standard-json flag. */
    standardJson: any;
    /** The path of the contract during compile-time. */
    compiledPath: string;
    /** The name of the contract. */
    name: string;
    /** Checks whether this contract is valid or not.
     * @returns true if no sources are missing or are invalid (malformed); false otherwise
     */
    isValid(): boolean;
    private sourceMapToStringMap;
    constructor(metadata: any, solidity: SourceMap, missing: any, invalid: StringMap);
    /**
     * Returns the contract path and name as specified in the metadata provided during construction.
     */
    private getPathAndName;
    /**
     * Returns a JSON object compatible with solc --standard-json.
     *
     * @param useOriginalSettings If true, the returned object will contain settings as specified
     * during the original compilation; otherwise no settings will be imposed.
     */
    getStandardJson(useOriginalSettings?: boolean): any;
    /**
     * Constructs the message to be displayed in case of a successful finding
     * of source files related to the provided metadata file.
     */
    private composeSuccessMessage;
    /**
     * Constructs the message to be displayed in case the contract is not valid.
     * The structure of the message is as follows:
     *
     * path: name
     *
     *   missing sources
     *
     *   invalid sources
     *
     *   number of found sources
     */
    private composeErrorMessage;
}
