"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationService = exports.PathBuffer = void 0;
var web3_1 = __importDefault(require("web3"));
var adm_zip_1 = __importDefault(require("adm-zip"));
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var CheckedContract_1 = __importDefault(require("./CheckedContract"));
/**
 * Regular expression matching metadata nested within another json.
 */
var NESTED_METADATA_REGEX = /"{\\"compiler\\":{\\"version\\".*?},\\"version\\":1}"/;
var PathBuffer = /** @class */ (function () {
    function PathBuffer(buffer, path) {
        this.buffer = buffer;
        this.path = path;
    }
    return PathBuffer;
}());
exports.PathBuffer = PathBuffer;
var PathContent = /** @class */ (function () {
    function PathContent(content, path) {
        this.content = content;
        this.path = path;
    }
    return PathContent;
}());
var ValidationService = /** @class */ (function () {
    /**
     * @param logger a custom logger that logs all errors; undefined or no logger provided turns the logging off
     */
    function ValidationService(logger) {
        this.logger = logger;
    }
    ValidationService.prototype.checkPaths = function (paths, ignoring) {
        var _this = this;
        var files = [];
        paths.forEach(function (path) {
            if (fs_1.default.existsSync(path)) {
                _this.traversePathRecursively(path, function (filePath) {
                    var fullPath = path_1.default.resolve(filePath);
                    var file = new PathBuffer(fs_1.default.readFileSync(filePath), fullPath);
                    files.push(file);
                });
            }
            else if (ignoring) {
                ignoring.push(path);
            }
        });
        return this.checkFiles(files);
    };
    ValidationService.prototype.checkFiles = function (files) {
        var _this = this;
        var inputFiles = this.findInputFiles(files);
        // const sanitizedFiles = this.sanitizeInputFiles(inputFiles);
        var parsedFiles = inputFiles.map(function (pathBuffer) { return new PathContent(pathBuffer.buffer.toString(), pathBuffer.path); });
        var metadataFiles = this.findMetadataFiles(parsedFiles);
        var checkedContracts = [];
        var errorMsgMaterial = [];
        metadataFiles.forEach(function (metadata) {
            var _a = _this.rearrangeSources(metadata, parsedFiles), foundSources = _a.foundSources, missingSources = _a.missingSources, invalidSources = _a.invalidSources;
            var checkedContract = new CheckedContract_1.default(metadata, foundSources, missingSources, invalidSources);
            checkedContracts.push(checkedContract);
            if (!checkedContract.isValid()) {
                errorMsgMaterial.push(checkedContract.info);
            }
        });
        if (errorMsgMaterial.length) {
            var msg = errorMsgMaterial.join("\n");
            if (this.logger)
                this.logger.error(msg);
        }
        return checkedContracts;
    };
    /**
     * Traverses the given files, unzipping any zip archive.
     *
     * @param files the array containing the files to be checked
     * @returns an array containing the provided files, with any zips being unzipped and returned
     */
    ValidationService.prototype.findInputFiles = function (files) {
        var inputFiles = [];
        for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
            var file = files_1[_i];
            if (this.isZip(file.buffer)) {
                this.unzip(file, files);
            }
            else {
                inputFiles.push(file);
            }
        }
        return inputFiles;
    };
    /**
     * Checks whether the provided file is in fact zipped.
     * @param file the buffered file to be checked
     * @returns true if the file is zipped; false otherwise
     */
    ValidationService.prototype.isZip = function (file) {
        try {
            new adm_zip_1.default(file);
            return true;
        }
        catch (err) { }
        return false;
    };
    /**
     * Unzips the provided file buffer to the provided array.
     *
     * @param zippedFile the buffer containin the zipped file to be unpacked
     * @param files the array to be filled with the content of the zip
     */
    ValidationService.prototype.unzip = function (zippedFile, files) {
        var timestamp = Date.now().toString();
        var tmpDir = "tmp-unzipped-" + timestamp;
        new adm_zip_1.default(zippedFile.buffer).extractAllTo(tmpDir);
        this.traversePathRecursively(tmpDir, function (filePath) {
            var file = new PathBuffer(fs_1.default.readFileSync(filePath), zippedFile.path);
            files.push(file);
        });
        this.traversePathRecursively(tmpDir, fs_1.default.unlinkSync, fs_1.default.rmdirSync);
    };
    ValidationService.prototype.sanitizeInputFiles = function (files) {
        var sanitizedFiles = [];
        if (!files.length) {
            var msg = 'Unable to extract any files. Your request may be misformatted ' +
                'or missing some content.';
            if (this.logger)
                this.logger.error(msg);
            throw new Error(msg);
        }
        for (var _i = 0, files_2 = files; _i < files_2.length; _i++) {
            var file = files_2[_i];
            var content = file.buffer.toString();
            /*try {
                const val = JSON.parse(file.buffer.toString());
                const type = Object.prototype.toString.call(val);

                // JSON formatted metadata or Stringified metadata
                if (type === '[object Object]') {
                    content = JSON.stringify(val);
                } else if (Array.isArray(val)) {
                    content = JSON.stringify(val[0]);
                } else {
                    content = val;
                }

            } catch (err) {
                content = file.buffer.toString();          // Solidity files
            }*/
            sanitizedFiles.push(new PathContent(content, file.path));
        }
        return sanitizedFiles;
    };
    /**
     * Selects metadata files from an array of files that may include sources, etc
     * @param  {string[]} files
     * @return {string[]}         metadata
     */
    ValidationService.prototype.findMetadataFiles = function (files) {
        var metadataCollection = [];
        for (var _i = 0, files_3 = files; _i < files_3.length; _i++) {
            var file = files_3[_i];
            var metadata = this.extractMetadataFromString(file.content);
            if (!metadata) {
                var matchRes = file.content.match(NESTED_METADATA_REGEX);
                if (matchRes) {
                    metadata = this.extractMetadataFromString(matchRes[0]);
                }
            }
            if (metadata) {
                metadataCollection.push(metadata);
            }
        }
        if (!metadataCollection.length) {
            var msg = "Metadata file not found. Did you include \"metadata.json\"?";
            if (this.logger)
                this.logger.error(msg);
            throw new Error(msg);
        }
        return metadataCollection;
    };
    /**
     * Validates metadata content keccak hashes for all files and
     * returns mapping of file contents by file name
     * @param  {any}       metadata
     * @param  {string[]}  files    source files
     * @return foundSources, missingSources, invalidSources
     */
    ValidationService.prototype.rearrangeSources = function (metadata, files) {
        var foundSources = {};
        var missingSources = {};
        var invalidSources = {};
        var byHash = this.storeByHash(files);
        for (var fileName in metadata.sources) {
            var sourceInfo = metadata.sources[fileName];
            var file = new PathContent(undefined);
            file.content = sourceInfo.content;
            var hash = sourceInfo.keccak256;
            if (file.content) {
                if (web3_1.default.utils.keccak256(file.content) != hash) {
                    var msg = "The calculated and the provided hash values don't match.";
                    invalidSources[fileName] = msg;
                    continue;
                }
            }
            else {
                file = byHash.get(hash);
            }
            if (file && file.content) {
                foundSources[fileName] = file;
            }
            else {
                missingSources[fileName] = { keccak256: hash, urls: sourceInfo.urls };
            }
        }
        return { foundSources: foundSources, missingSources: missingSources, invalidSources: invalidSources };
    };
    /**
     * Generates a map of files indexed by the keccak hash of their content.
     *
     * @param  {string[]}  files Array containing sources.
     * @returns Map object that maps hash to PathContent.
     */
    ValidationService.prototype.storeByHash = function (files) {
        var byHash = new Map();
        for (var i in files) {
            var calculatedHash = web3_1.default.utils.keccak256(files[i].content);
            byHash.set(calculatedHash, files[i]);
        }
        return byHash;
    };
    ValidationService.prototype.extractMetadataFromString = function (file) {
        try {
            var obj = JSON.parse(file);
            if (this.isMetadata(obj)) {
                return obj;
            }
            // if the input string originates from a file where it was double encoded (e.g. truffle)
            obj = JSON.parse(obj);
            if (this.isMetadata(obj)) {
                return obj;
            }
        }
        catch (err) { }
        return null;
    };
    /**
     * A method that checks if the provided object was generated as a metadata file of a Solidity contract.
     * Current implementation is rather simplistic and may require further engineering.
     *
     * @param metadata the JSON to be checked
     * @returns true if the provided object is a Solidity metadata file; false otherwise
     */
    ValidationService.prototype.isMetadata = function (obj) {
        return (obj.language === "Solidity") &&
            !!obj.compiler;
    };
    /**
     * Applies the provided worker function to the provided path recursively.
     *
     * @param path the path to be traversed
     * @param worker the function to be applied on each file that is not a directory
     * @param afterDir the function to be applied on the directory after traversing its children
     */
    ValidationService.prototype.traversePathRecursively = function (path, worker, afterDirectory) {
        var _this = this;
        if (!fs_1.default.existsSync(path)) {
            var msg = "Encountered a nonexistent path: " + path;
            if (this.logger) {
                this.logger.error(msg);
            }
            throw new Error(msg);
        }
        var fileStat = fs_1.default.lstatSync(path);
        if (fileStat.isFile()) {
            worker(path);
        }
        else if (fileStat.isDirectory()) {
            fs_1.default.readdirSync(path).forEach(function (nestedName) {
                var nestedPath = path_1.default.join(path, nestedName);
                _this.traversePathRecursively(nestedPath, worker, afterDirectory);
            });
            if (afterDirectory) {
                afterDirectory(path);
            }
        }
    };
    return ValidationService;
}());
exports.ValidationService = ValidationService;
//# sourceMappingURL=ValidationService.js.map